//+------------------------------------------------------------------+
//|                                               MoneyManagment.mqh |
//|                                 Copyright 2012, Evgeniy Trofimov |
//|                        https://login.mql5.com/ru/users/EvgeTrofi |
//+------------------------------------------------------------------+
//#include <EvgeTrofi\MoneyManagment.mqh>
//      
enum TYPE_LOT{
   LOT_FIXED=0,     //  .   ,    inValue
   LOT_RISK=1,      //       ( )
   LOT_CURRENCY=2   //       
};
class CMoneyManagment{
private:
public:
                    // CMoneyManagment(void){};
                    //~CMoneyManagment(void){};
       double PrCur(string inSymbol="", int inType=-1);
       double Currency_Transfer(double Value, string CurrentCurrency="EUR", string NeedCurrency="USD", int inType=-1);
       double GetMargin(string inSymbol="", int inType=-1);
       double TickValue(string inSymbol="", int inType=-1);
       double Lot(double inValue, TYPE_LOT inTypeLot=LOT_FIXED, string inSymbol="", int inType=-1);
       double LotNormalize(double inValue, string inSymbol="", int inType=-1);
       double LotSL(double inValue, int inSL, TYPE_LOT inTypeLot=LOT_FIXED, string inSymbol="", int inType=-1);
       double GetCurrentLot(string inSymbol="", ulong inMagic=0);
       double GetCurrentProfit(string inSymbol="", ulong inMagic=0);
       bool   LotIsKiller(double inLot, string inSymbol, int inType);
};
//+------------------------------------------------------------------+
//     ,      ,  
//        inValue.
//inValue -  ,       
//inType - (0 - , 1 - , -1 -      Bid  Ask)
//inTypeLot -    (.  )
//inSymbol -  
double CMoneyManagment::Lot(double inValue,TYPE_LOT inTypeLot=0,string inSymbol="",int inType=-1){
   if(inSymbol=="") inSymbol=Symbol();
   double res=inValue;
   double calc_margin=0;
   if(inTypeLot>0){
      calc_margin=GetMargin(inSymbol, inType);
      if(calc_margin<=0) {
         Print(__FUNCTION__,"    !");
         return(-1);
      }
      if(inTypeLot==LOT_RISK) res = AccountInfoDouble(ACCOUNT_EQUITY) * res / calc_margin / 100;
      if(inTypeLot==LOT_CURRENCY) res = res / calc_margin;
   }
   return(LotNormalize(res,inSymbol,inType));
}//Lot()
//+------------------------------------------------------------------+
//     ,      ,  
//      StopLoss     .
//inValue -  ,       
//inSL -        StopLoss  
//inType - (0 - , 1 - , -1 -      Bid  Ask)
//inTypeLot -    (.  )
//inSymbol -  
double CMoneyManagment::LotSL(double inValue,int inSL,TYPE_LOT inTypeLot=0,string inSymbol="",int inType=-1){
   if(inSymbol=="") inSymbol=Symbol();
   double res=inValue;
   double calc_tv=0;
   if(inTypeLot>0){
      if(inSL<=0){
         Print(__FUNCTION__,"        StopLoss");
         return(-1);
      }
      calc_tv=TickValue(inSymbol, inType)*inSL;
      if(calc_tv<=0) {
         Print(__FUNCTION__,"     !");
         return(-1);
      }
      if(inTypeLot==LOT_RISK) res = AccountInfoDouble(ACCOUNT_EQUITY) * res / calc_tv / 100;
      if(inTypeLot==LOT_CURRENCY) res = res / calc_tv;
   }
   return(LotNormalize(res,inSymbol,inType));
}//LotSL()
//+------------------------------------------------------------------+
//  
double CMoneyManagment::LotNormalize(double inValue,string inSymbol="",int inType=-1){
   if(inSymbol=="") inSymbol=Symbol();
   double free_margin = AccountInfoDouble(ACCOUNT_FREEMARGIN);
   double calc_margin = GetMargin(inSymbol, inType);
   if(calc_margin<=0) return(-1);
   double res = inValue;
   if(free_margin < calc_margin * res){
      if(calc_margin * res > 0.5 * AccountInfoDouble(ACCOUNT_EQUITY)){
         Alert("   ,          50%!!!");
         return(-1);
      }
      res = free_margin * 0.9 / calc_margin;
      Print(__FUNCTION__,"   : ",DoubleToString(inValue,2)," -> ", DoubleToString(res, 2));
   }
   res = MathMax(res,SymbolInfoDouble(inSymbol, SYMBOL_VOLUME_MIN));
   res = MathMin(res,SymbolInfoDouble(inSymbol, SYMBOL_VOLUME_MAX));
   double lot_step = SymbolInfoDouble(inSymbol ,SYMBOL_VOLUME_STEP);
   int norm = (int)MathRound(NormalizeDouble(MathCeil(MathAbs(MathLog(lot_step)/MathLog(10.0))), 0));
   return(NormalizeDouble(res, norm));
}//LotNormalize()
//+------------------------------------------------------------------+
double CMoneyManagment::TickValue(string inSymbol="",int inType=-1){
//        ,        1 
//      1 .
   double res=0;
   if(inSymbol=="") inSymbol=Symbol();
   ENUM_SYMBOL_CALC_MODE Mode = (ENUM_SYMBOL_CALC_MODE)SymbolInfoInteger(inSymbol, SYMBOL_TRADE_CALC_MODE);   
   switch(Mode){
      case SYMBOL_CALC_MODE_FOREX:
         res = SymbolInfoDouble(inSymbol,SYMBOL_TRADE_TICK_VALUE);
         return(res);
      case SYMBOL_CALC_MODE_FUTURES:
         Print(__FUNCTION__,"    FUTURES  !");
         return(-1);
      case SYMBOL_CALC_MODE_CFD:
         res = SymbolInfoDouble(inSymbol,SYMBOL_TRADE_TICK_VALUE)
              *SymbolInfoDouble(inSymbol,SYMBOL_TRADE_TICK_SIZE)
              *SymbolInfoDouble(inSymbol,SYMBOL_TRADE_CONTRACT_SIZE)
              *AccountInfoInteger(ACCOUNT_LEVERAGE);
         res = Currency_Transfer(res, 
                                 SymbolInfoString(inSymbol, SYMBOL_CURRENCY_PROFIT), 
                                 AccountInfoString(ACCOUNT_CURRENCY),inType);
         return(res);
      default:
         Print(__FUNCTION__,"    ",StringSubstr(EnumToString(Mode),17),"  !");
         return(-1);
   }
   return(res);   
}//TickValue()
//+------------------------------------------------------------------+
double CMoneyManagment::PrCur(string inSymbol="",int inType=-1){
   //   (Ask -  , Bid -  )
   if(inSymbol=="") inSymbol=Symbol();
   MqlTick tick;
   if(!SymbolInfoTick(inSymbol, tick)) {
      Print(__FUNCTION__,"      "+inSymbol);
      return(-1);
   }
   if(inType == 0){ //
      return(tick.ask);
   }else if(inType == 1){ //
      return(tick.bid);
   }else{ // 
      return((tick.bid+tick.ask)/2);
   }
   return(0);
}//PrCur()
//+------------------------------------------------------------------+
double CMoneyManagment::Currency_Transfer(double Value,string CurrentCurrency="EUR",string NeedCurrency="USD", int inType=-1){
   //  :)
   //     Value   NeedCurrency
   int i;
   double res = Value;
   string name;
   string USD="USD";
   string FndCurrency="";
   //  :
   if(CurrentCurrency==NeedCurrency) return(Value);
   for(i=0; i<SymbolsTotal(true); i++){
      name = SymbolName(i,true);
      if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == CurrentCurrency &&
         SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == NeedCurrency){
         res = res / PrCur(name, inType);
         return(res);
      }else if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == NeedCurrency &&
               SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == CurrentCurrency){
         res = res * PrCur(name, inType);
         return(res);
      }
   }//Next i
   //        
   if(CurrentCurrency!=USD){
      for(i=0; i<SymbolsTotal(true); i++){
         name = SymbolName(i,true);
         if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == CurrentCurrency &&
            SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == USD){
            res = res / PrCur(name, inType); //    
            FndCurrency = USD;
            break;
         }else if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == USD &&
                  SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == CurrentCurrency){
            res = res * PrCur(name, inType); //    
            FndCurrency = USD;
            break;
         }
      }//Next i
      if(FndCurrency!=USD){
         Print(__FUNCTION__,"         "+CurrentCurrency);
         Print("  ' ' ,    ,    "+CurrentCurrency+ "  "+USD+".");
         return(-1);
      }
      //Print(" "+CurrentCurrency+" -> "+USD+"  "+name);
   }
   if(NeedCurrency==USD) return(res);
   for(i=0; i<SymbolsTotal(true); i++){
      name = SymbolName(i,true);
      if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == USD &&
         SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == NeedCurrency){
         res = res / PrCur(name, inType); //    
         FndCurrency = NeedCurrency;
         break;
      }else if(SymbolInfoString(name, SYMBOL_CURRENCY_PROFIT) == NeedCurrency &&
               SymbolInfoString(name, SYMBOL_CURRENCY_BASE) == USD){
         res = res * PrCur(name, inType); //    
         FndCurrency = NeedCurrency;
         break;
      }
   }//Next i
   //Print(" "+USD+" -> "+NeedCurrency+"  "+name);
   if(FndCurrency==NeedCurrency) return(res);
   Print(__FUNCTION__,"         "+NeedCurrency);
   Print("  ' ' ,    ,    "+NeedCurrency+ "  "+USD+".");
   return(-1);
}//Currency_Transfer()
//+------------------------------------------------------------------+
double CMoneyManagment::GetMargin(string inSymbol="", int inType=-1){
//    ,      1 .
   if(inSymbol=="") inSymbol=Symbol();
   //MqlTick last_tick;
   double res=0;
   if(inType==0){ //   
      res = SymbolInfoDouble(inSymbol, SYMBOL_MARGIN_LONG);
   }else{ //   
      res = SymbolInfoDouble(inSymbol, SYMBOL_MARGIN_SHORT);
   }
   res=res*SymbolInfoDouble(inSymbol, SYMBOL_TRADE_CONTRACT_SIZE);
   ENUM_SYMBOL_CALC_MODE Mode = (ENUM_SYMBOL_CALC_MODE)SymbolInfoInteger(inSymbol, SYMBOL_TRADE_CALC_MODE);
   switch(Mode){
      case SYMBOL_CALC_MODE_FOREX:
         res = Currency_Transfer(res, SymbolInfoString(inSymbol, SYMBOL_CURRENCY_MARGIN), AccountInfoString(ACCOUNT_CURRENCY),inType);
         return(res/AccountInfoInteger(ACCOUNT_LEVERAGE));
      case SYMBOL_CALC_MODE_FUTURES:
         Print(__FUNCTION__,"    FUTURES  !");
         return(-1);
         /*
         res = Currency_Transfer(SymbolInfoDouble(inSymbol, SYMBOL_MARGIN_INITIAL), 
                                 SymbolInfoString(inSymbol, SYMBOL_CURRENCY_MARGIN), 
                                 AccountInfoString(ACCOUNT_CURRENCY),inType);
         return(res/AccountInfoInteger(ACCOUNT_LEVERAGE));
         */
      case SYMBOL_CALC_MODE_CFD:
         //if(!SymbolInfoTick(inSymbol, last_tick)) return(-1);
         res = Currency_Transfer(res, 
                                 SymbolInfoString(inSymbol, SYMBOL_CURRENCY_MARGIN), 
                                 AccountInfoString(ACCOUNT_CURRENCY),inType);
         /*
         if(inType==0){
            res = res * last_tick.ask;
         }else if(inType==1){
            res = res * last_tick.bid;
         }else{
            res = res * (last_tick.ask+last_tick.bid)/2;
         }*/
         return(res*PrCur(inSymbol, inType));
      default:
         Print(__FUNCTION__,"    ",StringSubstr(EnumToString(Mode),17),"  !");
         return(-1);
   }
   return(res);
}//GetMargin()
//+------------------------------------------------------------------+
//          
//   ,     
double CMoneyManagment::GetCurrentLot(string inSymbol="",ulong inMagic=0){
   if(inSymbol=="") inSymbol=Symbol();
   if(!PositionSelect(inSymbol)) return(0);
   ulong t; // 
   double lot=0;
   ENUM_DEAL_TYPE trend;
   HistorySelectByPosition(PositionGetInteger(POSITION_IDENTIFIER));
   for(int i=0; i<HistoryDealsTotal(); i++){
      t = HistoryDealGetTicket(i);
      if(inMagic>0){
         if(HistoryDealGetInteger(t,DEAL_MAGIC)!=inMagic) continue;
      }
      trend = (ENUM_DEAL_TYPE)HistoryDealGetInteger(t, DEAL_TYPE);
      if(trend==DEAL_TYPE_BUY){
         lot = lot + HistoryDealGetDouble(t, DEAL_VOLUME);
      }else if(trend==DEAL_TYPE_SELL){
         lot = lot - HistoryDealGetDouble(t, DEAL_VOLUME);
      }
   }//Next i
   return(lot);
}//GetCurrentLot()
//+------------------------------------------------------------------+
//      true,      inSymbol,
//  inType,  inLot    .
bool CMoneyManagment::LotIsKiller(double inLot,string inSymbol,int inType){
   if(!PositionSelect(inSymbol)) return(false);
   if(PositionGetInteger(POSITION_TYPE)==inType) return(false);
   if(PositionGetDouble(POSITION_VOLUME)==inLot) return(true);
   return(false);
}//LotIsKiller()
//+------------------------------------------------------------------+
double CMoneyManagment::GetCurrentProfit(string inSymbol="",ulong inMagic=0){
   return(0);
}//GetCurrentProfit()